package com.gct.tools.etlcamelhuge.routeconfig;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.common.util.SGTUtil;
import com.gct.tools.etlcamelhuge.MQ.MessageProducer;
import com.gct.tools.etlcamelhuge.camelconfig.TianAnDatabaseProperties;
import com.gct.tools.etlcamelhuge.entity.DiagnoseMsg;
import com.gct.tools.etlcamelhuge.entity.TianAnGtPoint;
import org.apache.camel.Message;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.dataformat.bindy.csv.BindyCsvDataFormat;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.*;
import java.lang.management.MemoryMXBean;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.sql.Clob;
import java.sql.SQLException;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * class name: CamelJDBCCofRealTimeConfiguration.java
 * 实时导数据类
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/4/14 下午3:16
 */
@Configuration
public class CamelJDBCCofRealTimeConfiguration {

    private static long sendMsgRunTime = 0;

    public Double min(String[] strings) {
        double[] doubles = new double[strings.length];
        for (int i = 0; i < strings.length; i++) {
            doubles[i] = Double.parseDouble(strings[i]);
        }
        OptionalDouble min = Arrays.stream(doubles).min();
        return min.isPresent() ? min.getAsDouble() : null;
    }


    //获取最大载荷
    public Double max(String[] strings) {
        double[] doubles = new double[strings.length];
        for (int i = 0; i < strings.length; i++) {
            doubles[i] = Double.parseDouble(strings[i]);
        }
        OptionalDouble max = Arrays.stream(doubles).max();
        return max.isPresent() ? max.getAsDouble() : null;
    }


    public String getDate() {
        return LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
    }


    /***
     * 异步发送 RunTime 消息到MQ中
     * */
    // @Async
    public void sendDataToRocketMQ(String wellName, String wellId, String prodDate, Double stroke_length, Double stroke_frequency, String sgt) {
        String orgId = "0";
        DiagnoseMsg diagnoseMsg = new DiagnoseMsg(wellId, wellName, orgId, prodDate, sgt, LocalDateTime.now().toString(), stroke_length, stroke_frequency);
//        System.out.println(diagnoseMsg);
        sendMsgRunTime++;
        producer.send(diagnoseMsg);
    }

    @Resource(name = "diagnoseMessageProducer")
    private MessageProducer producer;

    private String clobToString(Clob clob) {
        StringBuilder sb = new StringBuilder();
        if (clob != null) {
            try (Reader reader = clob.getCharacterStream()) {
                char[] buffer = new char[1024];
                int bytesRead;
                while ((bytesRead = reader.read(buffer)) != -1) {
                    sb.append(buffer, 0, bytesRead);
                }
            } catch (SQLException e) {
                throw new RuntimeException(e);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        return sb.toString();

    }

    @Autowired
    private TianAnDatabaseProperties tianAn;

    private static String postTokenBodyTemplate = "\"username\":\"%s\",\"password\":\"%s\"";

    private static String postGtDataBodyTemplate = "\"unit_name\":\"%s\",\"tags\":\"%s\"";

    private JSONObject sendPostRequest(String apiUrl, Map<String, String> headers, String requestBody) throws IOException {
        URL url = new URL(apiUrl);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        // 配置请求
        connection.setRequestMethod("POST");
        connection.setDoOutput(true);
        // 设置请求头
        for (Map.Entry<String, String> entry : headers.entrySet()) {
            connection.setRequestProperty(entry.getKey(), entry.getValue());
        }
        // 写入请求体
        try (OutputStream os = connection.getOutputStream()) {
            byte[] input = requestBody.getBytes(StandardCharsets.UTF_8);
            os.write(input, 0, input.length);
        }
        // 处理响应
        int responseCode = connection.getResponseCode();
        try (BufferedReader br = new BufferedReader(
                new InputStreamReader(
                        responseCode == 200 ? connection.getInputStream() : connection.getErrorStream(),
                        StandardCharsets.UTF_8
                )
        )) {
            StringBuilder response = new StringBuilder();
            String responseLine;
            while ((responseLine = br.readLine()) != null) {
                response.append(responseLine.trim());
            }
            JSONObject obj = new JSONObject();
            obj.put("responseCode", responseCode);
            obj.put("response", response.toString());
            return obj;
        }
    }

    private JSONObject sendGetRequest(String apiUrl, Map<String, String> queryParams, Map<String, String> headers) throws IOException {
        // 构建带查询参数的URL
        StringBuilder urlBuilder = new StringBuilder(apiUrl);
        if (queryParams != null && !queryParams.isEmpty()) {
            urlBuilder.append("?");
            boolean firstParam = true;
            for (Map.Entry<String, String> param : queryParams.entrySet()) {
                if (!firstParam) {
                    urlBuilder.append("&");
                }
                urlBuilder.append(param.getKey())
                        .append("=")
                        .append(param.getValue());
                firstParam = false;
            }
        }
        URL url = new URL(urlBuilder.toString());
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        // 配置请求
        connection.setRequestMethod("GET");
        // 设置请求头
        for (Map.Entry<String, String> entry : headers.entrySet()) {
            connection.setRequestProperty(entry.getKey(), entry.getValue());
        }
        // 处理响应
        int responseCode = connection.getResponseCode();
        try (BufferedReader br = new BufferedReader(
                new InputStreamReader(
                        responseCode == 200 ? connection.getInputStream() : connection.getErrorStream(),
                        StandardCharsets.UTF_8
                )
        )) {
            StringBuilder response = new StringBuilder();
            String responseLine;
            while ((responseLine = br.readLine()) != null) {
                response.append(responseLine.trim());
            }
            JSONObject obj = new JSONObject();
            obj.put("responseCode", responseCode);
            obj.put("response", response.toString());
            return obj;
        }
    }

    private String timestampToBeijingTime(long timestamp) {
        // 创建北京时间的时区（Asia/Shanghai）
        ZoneId beijingZone = ZoneId.of("Asia/Shanghai");
        // 将时间戳转换为 Instant（UTC 时间点）
        Instant instant = Instant.ofEpochMilli(timestamp);
        // 将 Instant 转换为北京时间的 LocalDateTime
        LocalDateTime localDateTime = instant.atZone(beijingZone).toLocalDateTime();
        // 定义日期时间格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 格式化为字符串
        return localDateTime.format(formatter);
    }

    @Bean
    public RouteBuilder routerBuilderRealTimeNew() {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                from("timer:mytimer-insert-runtime?period=1800000")
                        .to("direct:postToken")
                        .to("direct:getAccessWell")
                        .to("direct:getTianAnGtPointData")
                        .to("direct:querySingleWellDataFromTianAn");


                from("direct:postToken")
                        .routeId("postToken")
                        .setHeader("Content-Type", constant("application/x-www-form-urlencoded"))
                        .setHeader("CamelHttpMethod", constant("POST"))
                        .setBody(constant(String.format(postTokenBodyTemplate, tianAn.getUsername(), tianAn.getPassword())))
                        .to(tianAn.getUrl() + "/token")
                        .process(exchange -> {
                            String response = exchange.getIn().getBody(String.class);
                            JSONObject obj = JSONObject.parseObject(response);
                            if (!obj.getString("result").equals("ok"))
                                throw new RuntimeException(obj.getString("content"));
                            exchange.setProperty("access_token", obj.getJSONObject("content").getString("access_token"));
                            exchange.setProperty("token_type", obj.getJSONObject("content").getString("token_type"));
                        });

                from("direct:getAccessWell")
                        .routeId("getAccessWell")
                        .setBody(simple("select well_id ,sgt_last_time  from centralbase.sys_access_well_control  where  access_status='1'  "))
                        .to("jdbc:centralbase")
                        .process(exchange -> {
                            List<Map<String, Object>> body = exchange.getIn().getBody(List.class);
                            Map<String, String> sgtLastTimeMap = new HashMap<>();
                            body.forEach(map -> {
                                if (Objects.isNull(map.get("sgt_last_time"))) {
                                    map.put("sgt_last_time", LocalDateTime.now().minusHours(4).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                                }
                                sgtLastTimeMap.put(map.get("well_id").toString(), map.get("sgt_last_time").toString().substring(0, 19));
                            });
                            exchange.setProperty("sgtLastTimeMap", sgtLastTimeMap);
                        });

                BindyCsvDataFormat bindy = new BindyCsvDataFormat(com.gct.tools.etlcamelhuge.entity.TianAnGtPoint.class);
                bindy.setLocale("en_US");

                from("direct:getTianAnGtPointData")
                        .routeId("getTianAnGtPointData")
                        .setHeader("CamelFileName", constant("employees.xlsx")) // Excel文件名
                        .to("classpath:data/employees.xlsx") // 从resources/data目录读取文件
                        .log("读取Excel文件成功")
                        .convertBodyTo(String.class) // 转换为字符串
                        .unmarshal(bindy) // 使用Bindy解析CSV/Excel
                        .log("解析Excel内容完成")
                        .process(exchange -> {
                            List<TianAnGtPoint> list = exchange.getIn().getBody(List.class);
                            //对wellname做分类,存入Exchange属性
                            Map<String, List<TianAnGtPoint>> wellNameMap = new HashMap<>();
                            for (TianAnGtPoint tianAnGtPoint : list) {
                                wellNameMap.computeIfAbsent(tianAnGtPoint.getWellName(), k -> new ArrayList<>()).add(tianAnGtPoint);
                            }
                            exchange.setProperty("wellNameData", wellNameMap);
                            exchange.getIn().setBody("Excel解析完成，数据已存入exchange属性");
                        })
                        .log("Excel数据已分类后存入Exchange属性");

                from("direct:querySingleWellDataFromTianAn")
                        .routeId("querySingleWellDataFromTianAn")
                        .process(exchange -> {
                            Map<String, List<TianAnGtPoint>> wellNameData = exchange.getProperty("wellNameData", Map.class);

                            wellNameData.forEach((wellName, list) -> {
                                //对acqUnitId 在进行分类 功图与冲次数据
                                Map<String, List<TianAnGtPoint>> acqUnitMap = new HashMap<>();
                                for (TianAnGtPoint tianAnGtPoint : list) {
                                    acqUnitMap.computeIfAbsent(tianAnGtPoint.getAcqUnitId(), k -> new ArrayList<>()).add(tianAnGtPoint);
                                }

                                Map<String, String> rowMap = new HashMap<>();

                                acqUnitMap.forEach((unitName, tianAnGtPoints) -> {
                                    Map<String, String> headers = new HashMap<>();
                                    headers.put("Content-Type", "application/json");
                                    headers.put("Authorization", "Bearer " + exchange.getProperty("access_token"));

                                    StringBuilder tags = new StringBuilder();
                                    tianAnGtPoints.forEach(tianAnGtPoint -> {
                                        tags.append(tianAnGtPoint.getTag()).append(",");
                                    });
                                    tags.deleteCharAt(tags.length() - 1);

                                    Map<String, String> queryParams = new HashMap<>();
                                    queryParams.put("unit_name", unitName);
                                    queryParams.put("tags", tags.toString());

                                    try {
                                        JSONObject obj = sendGetRequest(tianAn.getUrl() + "/api_3rd/get_tags_datas", queryParams, headers);
                                        JSONArray res = obj.getJSONArray("response");
                                        for (int i = 0; i < res.size(); i++) {
                                            JSONObject resObj = res.getJSONObject(i);
                                            rowMap.put(resObj.getString("tagName"), resObj.getString("value"));
                                        }
                                        if (res.size() > 0) {
                                            String prodDate = timestampToBeijingTime(res.getJSONObject(0).getLong("timestamp"));
                                            rowMap.put("prod_date", prodDate);
                                        }
                                    } catch (IOException e) {
                                        throw new RuntimeException(e);
                                    }
                                });

                                log.info("wellName:{},rowData:{}", wellName, rowMap);
                                Map<String, String> sgtLastTimeMap = exchange.getProperty("sgtLastTimeMap", Map.class);

                                if (rowMap.containsKey("prod_date") && sgtLastTimeMap.containsKey(wellName)) {
                                    DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                                    LocalDateTime prodDate = LocalDateTime.parse(rowMap.get("prod_date"), df);
                                    LocalDateTime sgtLastTime = LocalDateTime.parse(sgtLastTimeMap.get(wellName), df);
                                    if (prodDate.isAfter(sgtLastTime)) {
                                        rowMap.put("sgt_last_time", sgtLastTimeMap.get(wellName));
                                        rowMap.put("wellName", wellName);
                                        exchange.getIn().setHeader("sgt_last_time", sgtLastTimeMap.get(wellName));
                                        exchange.getIn().setHeader("wellName", wellName);

                                        exchange.getIn().setBody(rowMap);
                                        exchange.getContext().createProducerTemplate().send("direct:saveToCentralbaseAndSendMessage", exchange);
                                    }
                                }
                            });
                        });

                from("direct:saveToCentralbaseAndSendMessage")
                        .routeId("saveToCentralbaseAndSendMessage")
                        .process(exchange -> {
                            Map<String, Object> rowMap = exchange.getIn().getBody(Map.class);
                            String wellName = rowMap.get("wellName").toString();
                            String prodDate = rowMap.get("prod_date").toString();
                            String frequency = rowMap.get("CHC").toString();
                            if (!StringUtils.isEmpty(rowMap.get("DYNACARD_DISPLACEMENT")) && !StringUtils.isEmpty(rowMap.get("DYNACARD_LOAD"))) {
                                String[] displacements = rowMap.get("DYNACARD_DISPLACEMENT").toString().split(",");//10 四舍五入
                                for (int i = 0; i < displacements.length; i++) {
                                    displacements[i] = Double.toString(Double.parseDouble(displacements[i]) / 10);
                                }
                                String[] disp_loads = rowMap.get("DYNACARD_LOAD").toString().split(",");
                                Double SUSP_MAX_LOAD = max(disp_loads);
                                Double SUSP_MIN_LOAD = min(disp_loads);
                                String sgt = "";
                                for (int i = 0; i < displacements.length; i++) {
                                    sgt = sgt + displacements[i] + "," + disp_loads[i] + ",";
                                }
                                String[] s = sgt.split(",");
                                String w = "";
                                for (int i = 0; i < s.length; i++) {
                                    w += new BigDecimal(Math.round(Double.parseDouble(s[i]) * 100)).stripTrailingZeros().toPlainString() + ",";
                                }
                                Double[][] doubles = SGTUtil.encodeToDoubleArray(w);
                                rowMap.put("sgt", SGTUtil.encodeToString(doubles));
                                rowMap.put("SUSP_MAX_LOAD", SUSP_MAX_LOAD);
                                rowMap.put("SUSP_MIN_LOAD", SUSP_MIN_LOAD);
                                double stroke = Arrays.stream(doubles)
                                        .mapToDouble(row -> row[0])
                                        .max()
                                        .orElse(Double.NEGATIVE_INFINITY);
                                rowMap.put("stroke", stroke);
                                log.info("wellName:{},rowData:{}", wellName, rowMap);
                                // sendDataToRocketMQ(wellName, wellName, prodDate, stroke, frequency, sgt);
                            }
                        })
                        .setBody(simple("insert into centralbase.cb_temp_well_mech_runtime(well_id,prod_date,SUSP_MAX_LOAD,SUSP_MIN_LOAD,sgt,stroke_length,stroke_frequency) " +
                                "select '${body[wellName]}','${body[prod_date]}','${body[SUSP_MAX_LOAD]}','${body[SUSP_MIN_LOAD]}','${body[sgt]}','${body[stroke]}','${body[CHC]}' " +
                                "where NOT EXISTS (SELECT * FROM centralbase.cb_temp_well_mech_runtime WHERE well_id = '${body[wellName]}' and  prod_date = '${body[DYNA_CREATE_TIME]}' )"))
                        .to("jdbc:centralbase")
                        .setBody(simple("update centralbase.sys_access_well_control set sgt_last_time = '${header.sgt_last_time}' where well_id ='${header.well_id}' "))
                        .to("jdbc:centralbase")
                        .end();


                from("timer:mytimer-update-avg-mech_daily?period=10800000")
                        .routeId("update-avg-mech_daily")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        })
                        .setBody(simple("select well_id,avg(stroke_length) stroke_length  ,avg(stroke_frequency) stroke_frequency from centralbase.cb_temp_well_mech_runtime where prod_date::date='${header.date}' group by well_id"))
                        .to("jdbc:centralbase")
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            if (aRow.get("stroke_length") != null && aRow.get("stroke_frequency") != null) {
                                double stroke_length = Double.parseDouble(aRow.get("stroke_length").toString());
                                double stroke_frequency = Double.parseDouble(aRow.get("stroke_frequency").toString());
                                BigDecimal bd = new BigDecimal(stroke_length);
                                double stroke_lengt1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                BigDecimal bd1 = new BigDecimal(stroke_frequency);
                                double stroke_frequency1 = bd1.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                aRow.put("strokeLength", stroke_lengt1);
                                aRow.put("strokeFrequency", stroke_frequency1);
                            }
                        })
                        .setBody(simple("update centralbase.cb_temp_well_mech_daily set stroke_length='${body[strokeLength]}' ,stroke_frequency ='${body[strokeFrequency]}' where well_id = '${body[well_id]}' and prod_date::date='${header.date}' "))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-avg-mech_daily ->  centralbase.cb_temp_well_mech_daily update data failed")
                        .end();

            }
        };
    }

    @Bean
    public RouteBuilder routeBuilderWithRealTime() {
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                from("timer:mytimer-insert-runtime?period=1800000")
                        .routeId("insert-runtime")
                        .setBody(simple("select well_id ,sgt_last_time  from centralbase.sys_access_well_control  where  access_status='1'  "))
                        .to("jdbc:centralbase")
                        .split(body()).process(exchange -> {
                            HashMap body = exchange.getIn().getBody(HashMap.class);
                            exchange.getIn().setHeader("well_id", body.get("well_id"));
//                            System.out.println(body.get("well_id"));
                            if (Objects.isNull(body.get("sgt_last_time"))) body.put("sgt_last_time"
                                    , LocalDateTime.now().minusHours(4).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                            exchange.getIn().setHeader("sgt_last_time", body.get("sgt_last_time").toString().substring(0, 19));
                        })
                        .setBody(simple("select WELL_ID,WELL_NAME,DYNA_CREATE_TIME,CHECK_DATE,DISPLACEMENT,DISP_LOAD,STROKE,FREQUENCY,SUSP_MAX_LOAD,SUSP_MIN_LOAD" +
                                " from DEFAULT_GONGTU where CHECK_DATE >= to_date('${header.sgt_last_time}','yyyy-mm-dd hh24:mi:ss') " +
                                "and well_id='${header.well_id}' and  rownum<50 "))
                        .to("jdbc:oracle")
                        .process(exchange -> {
                            sendMsgRunTime = 0;
                        })
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            String prod_date = aRow.get("DYNA_CREATE_TIME").toString().split("\\+")[0];
                            aRow.put("DYNA_CREATE_TIME", prod_date);
                            aRow.put("sgt", "");
                            //swap table error design column
                            Object tempObj = aRow.get("STROKE");
                            aRow.put("STROKE", aRow.get("FREQUENCY"));
                            aRow.put("FREQUENCY", tempObj);
                            if (!StringUtils.isEmpty(aRow.get("DISPLACEMENT")) && !StringUtils.isEmpty(aRow.get("DISP_LOAD"))) {
                                String[] displacements = clobToString((Clob) aRow.get("DISPLACEMENT")).split(",");//10 四舍五入
                                for (int i = 0; i < displacements.length; i++) {
                                    displacements[i] = Double.toString(Double.parseDouble(displacements[i]) / 10);
                                }
//                                System.out.println("disp="+Arrays.deepToString(displacements));
                                String[] disp_loads = clobToString((Clob) aRow.get("DISP_LOAD")).split(",");
                                Double SUSP_MAX_LOAD = max(disp_loads);
                                Double SUSP_MIN_LOAD = min(disp_loads);
                                String sgt = "";
                                for (int i = 0; i < displacements.length; i++) {
                                    sgt = sgt + displacements[i] + "," + disp_loads[i] + ",";
                                }
                                String[] s = sgt.split(",");
                                String w = "";
                                for (int i = 0; i < s.length; i++) {
                                    w += new BigDecimal(Math.round(Double.parseDouble(s[i]) * 100)).stripTrailingZeros().toPlainString() + ",";
                                }
                                Double[][] doubles = SGTUtil.encodeToDoubleArray(w);
                                aRow.put("sgt", SGTUtil.encodeToString(doubles));
                                aRow.put("SUSP_MAX_LOAD", SUSP_MAX_LOAD);
                                aRow.put("SUSP_MIN_LOAD", SUSP_MIN_LOAD);
                            }
                            //对于位移没有数据,所有数据都在载荷中的特殊数据做特别处理
                            if (StringUtils.isEmpty(aRow.get("DISPLACEMENT")) && !StringUtils.isEmpty(aRow.get("DISP_LOAD"))) {
                                String DISP_LOAD = aRow.get("DISP_LOAD").toString().replaceAll(";", ",");
                                Double[][] doubles = SGTUtil.encodeToDoubleArray(DISP_LOAD);
                                aRow.put("sgt", SGTUtil.encodeToString(doubles));
                                String[] split = DISP_LOAD.split(",");
                                List<String> list = new ArrayList<>();
                                for (int i = 0; i < split.length; i++) {
                                    if (i % 2 != 0) {
                                        BigDecimal bigDecimal = new BigDecimal(split[i]).divide(new BigDecimal("100.0"));
                                        list.add(bigDecimal.toString());
                                    }
                                }
                                String[] loads = list.toArray(new String[0]);
                                Double SUSP_MAX_LOAD = null;
                                Double SUSP_MIN_LOAD = null;
                                if (loads.length > 0) {
                                    SUSP_MAX_LOAD = max(loads);
                                    SUSP_MIN_LOAD = min(loads);
                                }
                                aRow.put("SUSP_MAX_LOAD", SUSP_MAX_LOAD);
                                aRow.put("SUSP_MIN_LOAD", SUSP_MIN_LOAD);
                            }
                            aRow.putIfAbsent("STROKE", "0.0");
                            aRow.putIfAbsent("FREQUENCY", "0.0");
                            aRow.putIfAbsent("SUSP_MAX_LOAD", "0.0");
                            aRow.putIfAbsent("SUSP_MIN_LOAD", "0.0");
                            if (!StringUtils.isEmpty(aRow.get("FREQUENCY"))) {
                                BigDecimal bd = new BigDecimal(aRow.get("FREQUENCY").toString());
                                double FREQUENCY = bd.setScale(1, RoundingMode.HALF_UP).doubleValue();
                                aRow.put("FREQUENCY", FREQUENCY);
                            }
                            if (!StringUtils.isEmpty(aRow.get("STROKE"))) {
                                double stroke1 = Double.parseDouble(aRow.get("STROKE").toString());
                                BigDecimal bd = new BigDecimal(stroke1);
                                double STROKE = bd.setScale(1, RoundingMode.HALF_UP).doubleValue();
                                aRow.put("STROKE", STROKE);
                            }
                            String wellName = aRow.get("WELL_NAME").toString();
                            String wellId = aRow.get("WELL_ID").toString();
                            String prodDate = aRow.get("DYNA_CREATE_TIME").toString().substring(0, 19);
                            Double strokeLength = Double.valueOf(aRow.get("STROKE").toString());
                            Double strokeFrequency = Double.valueOf(aRow.get("FREQUENCY").toString());
                            String sgt = aRow.get("sgt").toString();
//                            System.out.println(sgt);
                            in.setHeader("sgt_last_time", prodDate);
                            in.setHeader("well_id", wellId);
                            sendDataToRocketMQ(wellName, wellId, prodDate, strokeLength, strokeFrequency, sgt);
                        })
                        .setBody(simple("insert into centralbase.cb_temp_well_mech_runtime(well_id,prod_date,SUSP_MAX_LOAD,SUSP_MIN_LOAD,sgt,stroke_length,stroke_frequency) " +
                                "select '${body[WELL_ID]}','${body[DYNA_CREATE_TIME]}','${body[SUSP_MAX_LOAD]}','${body[SUSP_MIN_LOAD]}','${body[sgt]}','${body[STROKE]}','${body[FREQUENCY]}' " +
                                "where NOT EXISTS (SELECT * FROM centralbase.cb_temp_well_mech_runtime WHERE well_id = '${body[WELL_ID]}' and  prod_date = '${body[DYNA_CREATE_TIME]}' )"))
                        .to("jdbc:centralbase")
                        .setBody(simple("update centralbase.sys_access_well_control set sgt_last_time = '${header.sgt_last_time}' where well_id ='${header.well_id}' "))
                        .to("jdbc:centralbase")
                        .end();

                from("timer:mytimer-update-avg-mech_daily?period=10800000")
                        .routeId("update-avg-mech_daily")
                        .process(exchange -> {
                            Message in = exchange.getIn();
                            in.setHeader("date", getDate());
                        })
                        .setBody(simple("select well_id,avg(stroke_length) stroke_length  ,avg(stroke_frequency) stroke_frequency from centralbase.cb_temp_well_mech_runtime where prod_date::date='${header.date}' group by well_id"))
                        .to("jdbc:centralbase")
                        .split(body()).process(exchange -> {
                            Message in = exchange.getIn();
                            HashMap<String, Object> aRow = in.getBody(HashMap.class);
                            if (aRow.get("stroke_length") != null && aRow.get("stroke_frequency") != null) {
                                double stroke_length = Double.parseDouble(aRow.get("stroke_length").toString());
                                double stroke_frequency = Double.parseDouble(aRow.get("stroke_frequency").toString());
                                BigDecimal bd = new BigDecimal(stroke_length);
                                double stroke_lengt1 = bd.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                BigDecimal bd1 = new BigDecimal(stroke_frequency);
                                double stroke_frequency1 = bd1.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                                aRow.put("strokeLength", stroke_lengt1);
                                aRow.put("strokeFrequency", stroke_frequency1);
                            }
                        })
                        .setBody(simple("update centralbase.cb_temp_well_mech_daily set stroke_length='${body[strokeLength]}' ,stroke_frequency ='${body[strokeFrequency]}' where well_id = '${body[well_id]}' and prod_date::date='${header.date}' "))
                        .doTry()
                        .to("jdbc:centralbase")
                        .doCatch(Exception.class)
                        .log("${header.date}" + " routeId:update-avg-mech_daily ->  centralbase.cb_temp_well_mech_daily update data failed")
                        .end();
            }

        };
    }
}
