package com.gct.tools.etlcamelhuge;

import com.gct.tools.etlcamelhuge.MQ.MessageProducer;
import com.gct.tools.etlcamelhuge.entity.DiagnoseMsg;
import com.gct.tools.etlcamelhuge.routeconfig.CamelJDBCConfiguration;
import lombok.Data;
import lombok.val;
import org.json.JSONObject;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.test.context.junit4.SpringRunner;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = EtlCamelHugeApplication.class)
class EtlCamelHugeApplicationTests {

    @Resource(name = "oracle")
    DataSource oracle;

    @Resource(name = "centralbase")
    DataSource centralbase;
    @Qualifier("diagnosis")
    @Autowired
    private DataSource diagnosis;

    @Test
    public void testOracle() {
        JdbcTemplate template = new JdbcTemplate(oracle);
        String sql = String.format("select WELL_ID,well_common_name,org_id from " +
                "zy_dxgw.pc_dev_well_attr_info where well_purpose_name='采油井'");
        List<Map<String, Object>> maps = template.queryForList(sql);
        maps.get(0).forEach((k, v) -> {
            System.out.printf("k: %s, v: %s\n", k, v);
        });
    }

    @Test
    public void orgCreate() {
        JdbcTemplate oracleTemplate = new JdbcTemplate(oracle);
        final JdbcTemplate centralbaseTemplate = new JdbcTemplate(centralbase);
        String sql = String.format("select * from zy_dxgw.V_PC_ORGANIZATION_T where CANTON = '重油公司'");
        List<Map<String, Object>> results = oracleTemplate.queryForList(sql);

        Map<String, Map<String, Object>> parentObjMap = new HashMap<>();
        Map<String, Map<String, Object>> orgIdMap = new HashMap<>();
        AtomicInteger a = new AtomicInteger(1);
        results.forEach(rowMap -> {
            rowMap.put("org_id_map_int", a.getAndIncrement());
            orgIdMap.put(rowMap.get("ORG_ID").toString(), rowMap);
        });
        results.forEach(rowMap -> {
            parentObjMap.put(rowMap.get("ORG_ID").toString(), orgIdMap.get(rowMap.get("PARENT_ID")));
        });

        results.forEach(rowMap -> {
            Stack<String> orgPreStack = new Stack<>();
            Map<String, Object> parentObj = rowMap;

            while ((parentObj = parentObjMap.get(parentObj.get("ORG_ID").toString())) != null) {
                orgPreStack.push(parentObj.get("ORG_NAME").toString());
            }
            StringBuilder orgIdPre = new StringBuilder();
            while (!orgPreStack.isEmpty()) {
                orgIdPre.append(orgPreStack.pop()).append("@");
            }
            orgIdPre.append(rowMap.get("ORG_NAME"));
            String orgIdPreStr = orgIdPre.toString();
            rowMap.put("ORG_ID_PRE", orgIdPreStr);

            String sqlInsertOrg = String.format("insert into centralbase.cb_pc_organization(" +
                            "org_id,org_name,org_code,org_level,parent_id,org_id_pre) " +
                            "values('%s','%s','%s','%s','%s','%s')", rowMap.get("org_id_map_int"), rowMap.get("ORG_NAME"), rowMap.get("ORG_CODE"),
                    orgIdPreStr.split("@").length
                    , Objects.nonNull(orgIdMap.get(rowMap.get("PARENT_ID").toString())) ? orgIdMap.get(rowMap.get("PARENT_ID").toString()).get("org_id_map_int") : "0"
                    , orgIdPreStr);
//            System.out.println(sqlInsertOrg);
            centralbaseTemplate.execute(sqlInsertOrg);   //excute finish, unused
        });

//        if(a.get()>1)return;
        //excute wellsource
        sql = String.format("select WELL_ID,well_common_name,org_id from " +
                "zy_dxgw.pc_dev_well_attr_info where well_purpose_name='采油井'");
        results = oracleTemplate.queryForList(sql);
        results.forEach(rowMap -> {
            String wellId = rowMap.get("WELL_ID").toString();
            String wellCommonName = rowMap.get("WELL_COMMON_NAME").toString();
            String wellLegalName = rowMap.get("WELL_COMMON_NAME").toString();
            String orgId = rowMap.get("ORG_ID").toString();
            String orgIdMapInt = orgIdMap.get(orgId).get("org_id_map_int").toString();
            String stationId = orgIdMap.get(orgId).get("ORG_ID_PRE").toString();
            String sqlInsertWellSource = String.format("insert into centralbase.cb_cd_well_source(well_id,well_common_name," +
                            "well_legal_name,org_id,station_id) values('%s','%s','%s','%s','%s')",
                    wellId, wellCommonName, wellLegalName, orgIdMapInt, stationId);
//            System.out.println(sqlInsertWellSource);
            centralbaseTemplate.execute(sqlInsertWellSource);
        });

    }

    @Test
    public void wellSourceCreate() {

    }

    @Resource(name = "diagnoseMessageProducer")
    private MessageProducer producer;

    @Test
    public void testMQ() {
        String wellId = "XJvgyrsWjj";
        String wellName = "金225";
        String orgId = "0";
        String prod_date = "2024-11-01 16:26:52";
        String sgt = "0,4446,2,4533,13,4616,30,4738,58,4895,95,5095,141,5309,197,5566,254,5801,326,6128,405,6437,493,6676,589,6511,683,6393,795,6354,912,6472,1034,6602,1162,6589,1295,6528,1420,6463,1562,6480,1707,6541,1855,6567,2006,6554,2144,6528,2299,6524,2456,6559,2613,6554,2770,6541,2912,6524,3069,6511,3224,6528,3378,6511,3529,6498,3677,6459,3809,6454,3951,6424,4089,6445,4222,6406,4349,6398,4458,6376,4573,6289,4679,6241,4778,6189,4867,6154,4938,6149,5008,6167,5066,6141,5112,6106,5145,6080,5163,6071,5169,5997,5163,5849,5141,5709,5105,5640,5055,5592,4997,5574,4918,5553,4828,5492,4726,5374,4612,5339,4500,5304,4367,5309,4224,5296,4075,5274,3920,5239,3775,5248,3611,5274,3444,5243,3274,5243,3104,5148,2951,5139,2783,5043,2617,4956,2452,4812,2290,4751,2131,4612,1990,4533,1839,4311,1692,4107,1549,3854,1412,3789,1292,3959,1162,4111,1038,4102,920,3941,810,3893,715,3959,616,4020,522,4054,436,4033,357,4024,293,4041,228,4085,170,4111,120,4137,79,4128,47,4163,25,4242,8,4342";
        String rcvDate = "2024-11-01T17:55:36.173";
        Double s = 3.1;
        Double n = 5.2;
        DiagnoseMsg msg = new DiagnoseMsg(wellId, wellName, orgId, prod_date, sgt, rcvDate, s, n);
        producer.send(msg);
    }
}
